@@ -4,10 +4,10 @@ import warnings
 
 from collections import defaultdict
 from copy import copy, deepcopy
+from threading import local
 
 from django.conf import settings
 from django import VERSION
-from django import template
 from django.core.cache import cache
 from django.db.models import signals
 from django.utils import six
@@ -15,10 +15,16 @@ from django.utils.http import urlquote
 from django.utils.translation import get_language
 from django.utils.encoding import python_2_unicode_compatible
 from django.template import Context
+from django.template.loader import get_template
+from django.template.base import (
+    FilterExpression, Lexer, Parser, Token, Variable, VariableDoesNotExist, TOKEN_BLOCK, UNKNOWN_SOURCE, TOKEN_TEXT,
+    TOKEN_VAR, VARIABLE_TAG_START)
 from django.template.defaulttags import url as url_tag
 
 from .utils import get_tree_model, get_tree_item_model, import_app_sitetree_module, generate_id_for
-from .settings import ALIAS_TRUNK, ALIAS_THIS_CHILDREN, ALIAS_THIS_SIBLINGS, ALIAS_THIS_PARENT_SIBLINGS, ALIAS_THIS_ANCESTOR_CHILDREN, UNRESOLVED_ITEM_MARKER
+from .settings import (
+    ALIAS_TRUNK, ALIAS_THIS_CHILDREN, ALIAS_THIS_SIBLINGS, ALIAS_THIS_PARENT_SIBLINGS, ALIAS_THIS_ANCESTOR_CHILDREN,
+    UNRESOLVED_ITEM_MARKER)
 
 
 MODEL_TREE_CLASS = get_tree_model()
@@ -42,6 +48,9 @@ _IDX_TPL = '%s|:|%s'
 # SiteTree app-wise object.
 _SITETREE = None
 
+_THREAD_LOCAL = local()
+_THREAD_LANG = 'sitetree_lang'
+
 
 def get_sitetree():
     """Returns SiteTree [singleton] object, implementing utility methods.
@@ -234,15 +243,15 @@ class LazyTitle(object):
         self.title = title
 
     def __str__(self):
-        my_lexer = template.Lexer(self.title, template.UNKNOWN_SOURCE)
+        my_lexer = Lexer(self.title, UNKNOWN_SOURCE)
         my_tokens = my_lexer.tokenize()
 
         # Deliberately strip off template tokens that are not text or variable.
         for my_token in my_tokens:
-            if my_token.token_type not in (template.TOKEN_TEXT, template.TOKEN_VAR):
+            if my_token.token_type not in (TOKEN_TEXT, TOKEN_VAR):
                 my_tokens.remove(my_token)
 
-        my_parser = template.Parser(my_tokens)
+        my_parser = Parser(my_tokens)
         return my_parser.parse().render(SiteTree.get_global_context())
 
     def __eq__(self, other):
@@ -314,7 +323,7 @@ class SiteTree(object):
         If so, returns i18n alias. If not, returns the initial alias.
         """
         if alias in _I18N_TREES:
-            current_language_code = get_language().replace('_', '-').split('-')[0]
+            current_language_code = self.lang_get().replace('_', '-').split('-')[0]
             i18n_tree_alias = '%s_%s' % (alias, current_language_code)
             trees_count = self.get_cache_entry('tree_aliases', i18n_tree_alias)
             if trees_count is False:
@@ -372,6 +381,10 @@ class SiteTree(object):
 
         return items
 
+    def current_app_is_admin(self):
+        """Returns boolean whether current application is Admin contrib."""
+        return self._global_context.current_app == 'admin'
+
     def get_sitetree(self, alias):
         """Gets site tree items from the given site tree.
         Caches result to dictionary.
@@ -380,10 +393,11 @@ class SiteTree(object):
         """
         self.cache_init()
         sitetree_needs_caching = False
-        if self._global_context.current_app != 'admin':
+        if not self.current_app_is_admin():
             # We do not need i18n for a tree rendered in Admin dropdown.
             alias = self.resolve_tree_i18n_alias(alias)
         sitetree = self.get_cache_entry('sitetrees', alias)
+
         if not sitetree:
             sitetree = MODEL_TREE_ITEM_CLASS.objects.select_related('parent', 'tree').\
                    filter(tree__alias__exact=alias).order_by('parent__sort_order', 'sort_order')
@@ -419,7 +433,7 @@ class SiteTree(object):
                                                item.access_permissions.select_related()])
             # Contextual properties.
             item.url_resolved = self.url(item)
-            if template.VARIABLE_TAG_START in item.title:
+            if VARIABLE_TAG_START in item.title:
                 item.title_resolved = LazyTitle(item.title)
             else:
                 item.title_resolved = item.title
@@ -454,18 +468,20 @@ class SiteTree(object):
         request path against URL of given tree item.
 
         """
-        if self._global_context.current_app == 'admin':
+        if self.current_app_is_admin():
             return None
 
         current_item = None
 
         if 'request' not in self._global_context:
             if settings.DEBUG:
-                raise SiteTreeError('Sitetree needs "django.core.context_processors.request" to be in TEMPLATE_CONTEXT_PROCESSORS in your settings file. If it is, check that your view pushes request data into the template.')
+                raise SiteTreeError(
+                    'Sitetree needs "django.core.context_processors.request" to be in TEMPLATE_CONTEXT_PROCESSORS '
+                    'in your settings file. If it is, check that your view pushes request data into the template.')
         else:
             # urlquote is an attempt to support non-ascii in url.
             current_url = urlquote(self._global_context['request'].path)
-            urls_cache = self.get_cache_entry('urls', tree_alias)
+            urls_cache = self.get_cache_entry('urls', '%s%s' % (tree_alias, self.lang_get()))
             if urls_cache:
                 for url_item in urls_cache:
                     urls_cache[url_item][1].is_current = False
@@ -477,6 +493,24 @@ class SiteTree(object):
 
         return current_item
 
+    @classmethod
+    def lang_get(cls):
+        """Returns language code for current thread.
+
+        :return:
+        """
+        return getattr(_THREAD_LOCAL, _THREAD_LANG, '') or cls.lang_init()
+
+    @classmethod
+    def lang_init(cls):
+        """Initializes and returns language code for current thread.
+
+        :return:
+        """
+        lang = get_language()
+        setattr(_THREAD_LOCAL, _THREAD_LANG, lang)
+        return lang
+
     def url(self, sitetree_item, context=None):
         """Resolves item's URL.
 
@@ -506,8 +540,9 @@ class SiteTree(object):
                     if isinstance(resolved, six.text_type):
                         if resolved.encode('ascii', 'ignore').decode('ascii') != resolved:
                             resolved = view_argument
-                        # URL parameters from site tree item should be concatenated with those from template.
-                    all_arguments.append('"%s"' % str(resolved))  # We enclose arg in double quotes as already resolved.
+
+                    # We enclose arg in double quotes as already resolved.
+                    all_arguments.append('"%s"' % str(resolved))
                 view_path = view_path[0].strip('"\' ')
 
             if VERSION >= (1, 5, 0):  # "new-style" url tag - consider sitetree named urls literals.
@@ -517,13 +552,14 @@ class SiteTree(object):
         else:
             url_pattern = str(sitetree_item.url)
 
-        tree_alias = sitetree_item.tree.alias
+        # i18n_patterns compatibility organized using compound cache key.
+        cache_key = '%s%s' % (sitetree_item.tree.alias, self.lang_get())
 
-        entry_from_cache = self.get_cache_entry('urls', tree_alias)
+        entry_from_cache = self.get_cache_entry('urls', cache_key)
         if not entry_from_cache:
             # Create 'cache_urls' for this tree.
             entry_from_cache = {}
-            self.set_cache_entry('urls', tree_alias, {})
+            self.set_cache_entry('urls', cache_key, {})
 
         if url_pattern in entry_from_cache:
             resolved_url = entry_from_cache[url_pattern][0]
@@ -531,7 +567,10 @@ class SiteTree(object):
             if sitetree_item.urlaspattern:
                 # Form token to pass to Django 'url' tag.
                 url_token = u'url %s as item.url_resolved' % url_pattern
-                url_tag(template.Parser(None), template.Token(token_type=template.TOKEN_BLOCK, contents=url_token)).render(context)
+                url_tag(
+                    Parser(None),
+                    Token(token_type=TOKEN_BLOCK, contents=url_token)
+                ).render(context)
 
                 # We make an anchor link from an unresolved URL as a reminder.
                 if not context['item.url_resolved']:
@@ -541,7 +580,7 @@ class SiteTree(object):
             else:
                 resolved_url = url_pattern
 
-            self.update_cache_entry_value('urls', tree_alias, {url_pattern: (resolved_url, sitetree_item)})
+            self.update_cache_entry_value('urls', cache_key, {url_pattern: (resolved_url, sitetree_item)})
 
         return resolved_url
 
@@ -553,6 +592,8 @@ class SiteTree(object):
         """
         # Current context we will consider global.
         self.set_global_context(context)
+        # Initialize language to use it in current thread.
+        self.lang_init()
         # Resolve tree_alias from the context.
         tree_alias = self.resolve_var(tree_alias)
         # Get tree.
@@ -573,7 +614,9 @@ class SiteTree(object):
         # Current item is unresolved, fail silently.
         if current_item is None:
             if settings.DEBUG:
-                raise SiteTreeError('Unable to resolve current sitetree item to get a `%s` for current page. Check whether there is an appropriate sitetree item defined for current URL.' % attr_name)
+                raise SiteTreeError(
+                    'Unable to resolve current sitetree item to get a `%s` for current page. Check whether '
+                    'there is an appropriate sitetree item defined for current URL.' % attr_name)
             return ''
         return getattr(current_item, attr_name, '')
 
@@ -704,11 +747,7 @@ class SiteTree(object):
         """
         # Resolve parent item and current tree alias.
         parent_item = self.resolve_var(parent_item, context)
-        tree_alias = parent_item.tree.alias
-        # Resolve tree_alias from the context.
-        tree_alias = self.resolve_var(tree_alias)
-        # Get tree.
-        tree_alias, tree_items = self.get_sitetree(tree_alias)
+        tree_alias, tree_items = self.get_sitetree(parent_item.tree.alias)
         # Mark path to current item.
         self.tree_climber(tree_alias, self.get_tree_current_item(tree_alias))
 
@@ -717,12 +756,12 @@ class SiteTree(object):
         tree_items = self.apply_hook(tree_items, '%s.children' % navigation_type)
         tree_items = self.update_has_children(tree_alias, tree_items, navigation_type)
 
-        my_template = template.loader.get_template(use_template)
+        my_template = get_template(use_template)
         context.update({'sitetree_items': tree_items})
         return my_template.render(context)
 
     def get_children(self, tree_alias, item):
-        if self._global_context.current_app != 'admin':
+        if not self.current_app_is_admin():
             # We do not need i18n for a tree rendered in Admin dropdown.
             tree_alias = self.resolve_tree_i18n_alias(tree_alias)
         return self.get_cache_entry('parents', tree_alias)[item]
@@ -743,7 +782,7 @@ class SiteTree(object):
         NB: We do not apply any filters to sitetree in admin app.
         """
         items_out = copy(items)
-        if self._global_context.current_app != 'admin':
+        if not self.current_app_is_admin():
             for item in items:
                 no_access = not self.check_access(item, self._global_context)
                 hidden_for_nav_type = navigation_type is not None and not getattr(item, 'in' + navigation_type, False)
@@ -787,14 +826,14 @@ class SiteTree(object):
         if context is None:
             context = self._global_context
 
-        if isinstance(varname, template.FilterExpression):
+        if isinstance(varname, FilterExpression):
             varname = varname.resolve(context)
         else:
             varname = varname.strip()
 
             try:
-                varname = template.Variable(varname).resolve(context)
-            except template.VariableDoesNotExist:
+                varname = Variable(varname).resolve(context)
+            except VariableDoesNotExist:
                 varname = varname
 
         return varname
